// (c) Copyright 2012 Cloudera, Inc. All rights reserved.
/*global Rickshaw: true */
define([
  "underscore",
  "cloudera/Util",
  'cloudera/common/Humanize',
  "cloudera/common/charts/rickshaw/Series",
  "cloudera/common/charts/rickshaw/HoverDetail",
  "cloudera/common/charts/rickshaw/FormatUtils",
  "cloudera/common/charts/rickshaw/AxisTime",
  "cloudera/common/charts/rickshaw/AxisY",
  "cloudera/common/Monitor",
  "rickshaw",
  "d3"
], function(_, Util, Humanize, Series, HoverDetail, FormatUtils, AxisTime, AxisY, Monitor) {

  /**
   * Renders one chart containing 1 or more time series.
   * options = {
   *   container:       (required) "the DOM or the selector of the chart's container element",
   *   width:           (required) the width of a single chart,
   *   height:          (required) the height of a single chart,
   *   yAxisWidth:      (required) the width of the y Axis.
   *   ymin:            (optional) the minimum Y value,
   *   ymax:            (optional) the maximum Y value,
   *   interpolation:   (optional) "rickshaw specific: linear || step-after",
   *   offset:          (optional) "rickshaw specific: value || zero",
   *   renderer:        (optional) "rickshaw specific: line || area || bar",
   *   showYAxis:       (optional) true|false whether the y Axis should be shown.
   *   showHoverDetail: (optional) true|false whether hover details should be shown.
   *   showLegend:      (optional) true|false whether a legend shoudl be shown.
   * }
   */
  return function(options) {
    var self = this, $container = $(options.container);

    self.createGraph = function(args) {
      return new Rickshaw.Graph(args);
    };

    self.createAxisX = function() {
      // Show X-Y axis.
      return new AxisTime({
        graph: self.graph
      } );
    };

    self.createAxisY = function(units) {
      return new AxisY({
        orientation: 'left',
        tickFormat: FormatUtils.getYAxisTickFormatFunction(units),
        pixelsPerTick: 30,
        graph: self.graph,
        element: $container.find(".yAxis")[0],
        width: options.yAxisWidth,
        height: options.height
      });
    };

    /**
     * Renders a single chart with 1 or more time series.
     */
    self.render = function(groupTs, monitor) {
      var logMonitor = false;
      if (arguments.length === 1) {
        monitor = new Monitor("Visualizer.render");
        logMonitor = true;
      }
      monitor.open("preparing args");
      var args = {
        element: $container.find(".chart")[0],
        series: self.computeSeries(groupTs),
        width: options.width,
        height: options.height,
        renderer: options.renderer || "line",
        offset: options.offset || "value",
        interpolation: options.interpolation || "linear",
        min: options.ymin,
        max: options.ymax
      };
      monitor.close("preparing args");
      self.graph = self.createGraph(args);
      // Show time series details on hover.
      if (options.showHoverDetail) {
        monitor.open("show hover detail");
        self.hoverDetail = new HoverDetail({
          graph: self.graph
        });
        monitor.close("show hover detail");
      }

      // Show a legend.
      if (options.showLegend) {
        self.legend = new Rickshaw.Graph.Legend({
            graph: self.graph,
            element: $container.find(".legend")[0]
        });
        self.legendToggle = new Rickshaw.Graph.Behavior.Series.Toggle({
            graph: self.graph,
            legend: self.legend
        });
        self.legendHighlight = new Rickshaw.Graph.Behavior.Series.Highlight({
            graph: self.graph,
            legend: self.legend
        });
        // This is hacky, but necessary. We do not want individual entries
        // in the legend to be sortable. We *should* get what we want
        // by not adding the Rickshaw.Graph.Behavior.Series.Order object above,
        // except that doesn't work. The Toggle object re-enables jQuery UI
        // sortable whether you want it or not, so we have to reach into the
        // legend and turn if off explicitly here.
        $container.find('.legend ul').sortable('disable');
      }

      var units = FormatUtils.getFirstUnits(groupTs);

      monitor.open("Creating x axis");
      self.xAxis = self.createAxisX();
      monitor.close("Creating x axis");

      monitor.open("Creating y axis");
      if (options.showYAxis) {
        self.yAxis = self.createAxisY(units);
        if (units) {
          self.yAxis.setUnits(units);
        }
      }
      monitor.close("Creating y axis");

      monitor.open("rendering chart");
      // Render the charts.
      self.graph.render();
      monitor.close("rendering chart");
      monitor.open("rending axises");
      if (options.showYAxis) {
        self.yAxis.render();
      }
      self.xAxis.render();
      monitor.close("rending axises");
      if (logMonitor) {
        monitor.log();
      }
    };

    /**
     * Converts data into series for rendering.
     */
    self.computeSeries = function(groupTs) {
      var palette = new Rickshaw.Color.Palette({
        scheme: Util.getChartColors()
      });

      var data = $.map(groupTs, function(ts) {
        var color;
        if (_.isArray(ts.data) && ts.data.length > 0 &&
          ts.data[0].type === "fake") {
          color = "#ccc";
        } else {
          color = palette.color();
        }
        return {
          data: ts.data,
          name: ts.metadata.label,
          color: color,
          units: ts.metadata.units
        };
      });
      return data;
    };

    /**
     * Updates the attributes of the graph without destroying it.
     */
    self.update = function(config) {
      if (self.graph) {
        self.graph.configure(config);
        self.graph.render();
        if (options.showYAxis) {
          self.yAxis.render();
        }
        self.xAxis.render();
      }
    };
  };
});
