// (c) Copyright 2012 Cloudera, Inc. All rights reserved.
define([
  'cloudera/health/HealthEvent',
  'cloudera/health/HealthUtil',
  'cloudera/events/EventsQuery',
  'cloudera/Util',
  'cloudera/common/Humanize',
  'cloudera/common/MagicStrings',
  'knockout',
  'komapping',
  'underscore'
], function(HealthEvent, HealthUtil, EventsQuery, Util, Humanize, strings, ko, komapping, _) {
  var DEFAULT_OFFSET = 0;
  var DEFAULT_LIMIT = 20;

  // Create a HealthEvent instance and then decorate it with
  // health-history-specific properties. This mostly includes I18n'ed text,
  // but also includes reorganizing the associated health check data for
  // easier data binding within the template.
  var createHealthEvents = function(serverData) {
    var previousEvent,
        sixtySeconds = 60 * 1000;
    return _.map(serverData, function(serverEvent) {
      var event = new HealthEvent(serverEvent);
      // Decorate the basic HealthEvent object with some properties that
      // are health history specific: tracking row expansion, humanizing
      // its timestamp, etc.

      // If the previous event happened more than 60 seconds in the future, show seconds in the formatted time.
      if (previousEvent && (previousEvent.timestamp - event.timestamp.getTime()) < sixtySeconds) {
        if (!previousEvent.showSeconds) {
          previousEvent.formattedTimestamp =
              Humanize.humanizeDateTimeAdaptable(previousEvent.timestamp, true);
          previousEvent.showSeconds = true;
        }

        event.showSeconds = true;
      }

      event.formattedTimestamp =
          Humanize.humanizeDateTimeAdaptable(event.timestamp, event.showSeconds);

      event.expanded = ko.observable(false);
      event.toggleExpansion = function() {
        event.expanded(!event.expanded());
      };
      event.healthText = HealthUtil.getHealthStatusText(event.health);
      // For every health test, construct an object with two attributes:
      // a descriptive short name for this health check and a string that
      // gives the status of this health check (e.g. bad? good?).
      event.healthTests = _.map(event.healthTestResults, function(healthTest) {
        return {
          'shortName': HealthUtil.getHealthCheckShortName(healthTest.testName),
          'status': HealthUtil.getHealthStatusFromEventCode(healthTest.eventCode),
          'content': healthTest.content
        };
      });
      // event.statusCount has keys corresponding to status
      // (bad, good, etc). The value for a key is the number of health
      // checks with this status in this event.
      // We need to display this to the user, so iterate through the
      // status counts and I18n the counts.
      event.formattedStatusCounts = _.object(
        _.map(event.statusCounts, function(statusCount, status) {
          return [status,
            HealthUtil.formatHealthChanges(status, statusCount)];
      }));
      event.showEvent = function() {
        $.publish('changeMarkerTime', [new Date(event.timestamp), true]);
      };
      previousEvent = event;
      return event;
    });
  };

  // The view model for the health history widget.
  var HealthHistoryViewModel = function(serverData) {
    var self = this;
    self.events = ko.observableArray(createHealthEvents(serverData));
    // Prevents a flash of unbound content. It never changes.
    // Signifies to the template that it is now bound so that the bound
    // content can display safely.
    self.loaded = ko.observable(true);

    self.update = function(updatedData) {
      self.events(createHealthEvents(updatedData));
    };
  };

  // Options:
  // * id: the ID of the HTML element with the table. For applying
  //   KO bindings.
  // * endTime: a Date object for the end of the query
  // * eventSearchUrl: the base URL to construct event searches from.
  // * healthCheckUrl: the URL of the health check status endpoint. When the
  //   user expands a particular row in the health history table, we want
  //   to show them a list of the health checks from that point in time.
  // * service: the name of the service we want health history for.
  // * role: the name of the role we want health history for.
  // * host: the name of the host we want health history for.
  var HealthHistory = function(options) {
    this.id = options.id;
    this.healthCheckUrl = options.healthCheckUrl;
    this.eventSearchUrl = options.eventSearchUrl;

    // Build filters for event search.
    var filters = [{
      propertyName: strings.events.attributes.category,
      compareType: strings.compareTypes.equalsMultiple,
      values: [strings.events.categories.healthCheck]
    }];

    var eventCodeMap = HealthUtil.eventCodeMap;
    // Change the query based on what was passed in. Are we querying a
    // role's health history? A service? Etc.
    if (options.role) {
      filters.push({
        propertyName: strings.events.attributes.role,
        compareType: strings.compareTypes.equalsMultiple,
        values: [options.role]
      });
      filters.push({
        propertyName: strings.events.attributes.eventCode,
        compareType: strings.compareTypes.equalsMultiple,
        values:[
          eventCodeMap.EV_ROLE_HEALTH_CHECK_BAD,
          eventCodeMap.EV_ROLE_HEALTH_CHECK_CONCERNING,
          eventCodeMap.EV_ROLE_HEALTH_CHECK_DISABLED,
          eventCodeMap.EV_ROLE_HEALTH_CHECK_GOOD,
          eventCodeMap.EV_ROLE_HEALTH_CHECK_UNKNOWN]
      });
    } else if (options.service) {
      filters.push({
        propertyName: strings.events.attributes.service,
        compareType: strings.compareTypes.equalsMultiple,
        values: [options.service]
      });
      filters.push({
        propertyName: strings.events.attributes.eventCode,
        compareType: strings.compareTypes.equalsMultiple,
        values:[
          eventCodeMap.EV_SERVICE_HEALTH_CHECK_BAD,
          eventCodeMap.EV_SERVICE_HEALTH_CHECK_CONCERNING,
          eventCodeMap.EV_SERVICE_HEALTH_CHECK_DISABLED,
          eventCodeMap.EV_SERVICE_HEALTH_CHECK_GOOD,
          eventCodeMap.EV_SERVICE_HEALTH_CHECK_UNKNOWN]
      });
    } else if (options.host) {
      filters.push({
        propertyName: strings.events.attributes.hosts,
        compareType: strings.compareTypes.equalsMultiple,
        values: [options.host]
      });
      filters.push({
        propertyName: strings.events.attributes.eventCode,
        compareType: strings.compareTypes.equalsMultiple,
        values:[
          eventCodeMap.EV_HOST_HEALTH_CHECK_BAD,
          eventCodeMap.EV_HOST_HEALTH_CHECK_CONCERNING,
          eventCodeMap.EV_HOST_HEALTH_CHECK_DISABLED,
          eventCodeMap.EV_HOST_HEALTH_CHECK_GOOD,
          eventCodeMap.EV_HOST_HEALTH_CHECK_UNKNOWN]
      });
    }

    // The start time for our query is the beginning of time.
    this.query = new EventsQuery(new Date(0), options.endTime,
      filters, DEFAULT_OFFSET, DEFAULT_LIMIT);
  };

  HealthHistory.prototype = {
    started: false,

    start: function() {
      if (this.started) {
        return;
      }
      this.started = true;
      // A couple of our methods need binding. Do it once here.
      this.onEventsResponse = _.bind(this.onEventsResponse, this);
      this.onMarkerDateChanged = _.bind(this.onMarkerDateChanged, this);
      // Subscribe to global events.
      this.subscribe();
      // Fetch event data immediately.
      this.getEvents();
    },

    getEvents: function() {
      this.query.execute(this.eventSearchUrl, {
        success: this.onEventsResponse
      });
    },

    onEventsResponse: function(data) {
      // If this is the first time we've run, instantiate the view model
      // and bind it to the template.
      if (!this.viewModel) {
        this.viewModel = new HealthHistoryViewModel(data);
        // Bind to the table we're supposed to modify.
        ko.applyBindings(this.viewModel, $('#' + this.id)[0]);
      } else {
        this.viewModel.update(data, this.query.startTime,
          this.query.endTime);
      }
    },

    onMarkerDateChanged: function(markerDate) {
      this.query.endTime = markerDate;
      this.query.offset = DEFAULT_OFFSET;
      this.query.limit = DEFAULT_LIMIT;
      this.getEvents();
    },

    subscribe: function() {
      var handle1 = $.subscribe('markerDateChanged',
        this.onMarkerDateChanged);
      this.subscriptionHandles = [handle1];
    },

    unsubscribe: function() {
      Util.unsubscribe(this);
    }
  };

  // Expose type for testing.
  HealthHistory.HealthHistoryViewModel = HealthHistoryViewModel;

  return HealthHistory;
});
